//-------------------------------------------------------------------------------------------------+
// GlobalMapViewer.cpp
// This RTC is used to show Global Map
//-------------------------------------------------------------------------------------------------+

#include "GlobalMapViewer.h"
#include <iostream>

#define WINDOW_NAME		"Global Map"

using namespace std;


//------------------------------------------------------------------------------------------------+
// Module specification
//------------------------------------------------------------------------------------------------+
static const char* GlobalMapViewer_spec[] =
{
    "implementation_id", "GlobalMapViewer",
    "type_name",         "GlobalMapViewer",
    "description",       "This RTC is used to show Global Path.",
    "version",           "1.0.0",
    "vendor",            "aisl.cs.tut.ac.jp",
    "category",          "Viewe",
    "activity_type",     "PERIODIC",
    "kind",              "DataFlowComponent",
    "max_instance",      "0",
    "language",          "C++",
    "lang_type",         "compile",
    // Configuration variables
    "conf.default.color_reverse", "0",
    "conf.default.map_type", "1",
    "conf.default.view_scale", "1.0",
    // Widget
    "conf.__widget__.color_reverse", "text",
    "conf.__widget__.map_type", "text",
    "conf.__widget__.view_scale", "text",
    ""
};


//-------------------------------------------------------------------------------------------------+
// constructor
//-------------------------------------------------------------------------------------------------+
GlobalMapViewer::GlobalMapViewer(RTC::Manager* manager)
  : RTC::DataFlowComponentBase(manager),
    m_robot_poseIn("robot_pose", m_robot_pose),
    m_posesIn("poses", m_poses),
    m_pathIn("path", m_path),
    m_personsIn("persons", m_persons),
    m_AbsoluteMapServicePort("AbsoluteMapService"),
    m_AreaInfoServicePort("AreaInfoService")
{
	cerr << "------------------------------------------------------------+\n"
		 << " GlobalMapViewer Component (Ver. 3rd February 2012)\n"
		 << "------------------------------------------------------------+\n\n";
}


//-------------------------------------------------------------------------------------------------+
// destructor
//-------------------------------------------------------------------------------------------------+
GlobalMapViewer::~GlobalMapViewer(){

	// EBhEj
	cvDestroyWindow( WINDOW_NAME );

	if( convert ) delete convert;
}


//-------------------------------------------------------------------------------------------------+
// vON̏֐
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapViewer::onInitialize(){

	// Set InPort buffers
	addInPort("robot_pose", m_robot_poseIn);
	addInPort("poses", m_posesIn);
	addInPort("path", m_pathIn);
	addInPort("persons", m_personsIn);

	// Set service consumers to Ports
	m_AbsoluteMapServicePort.registerConsumer("AbsoluteMapService", "MRFC::AbsoluteMapService", m_AbsoluteMapService);
	m_AreaInfoServicePort.registerConsumer("AreaInfoService", "TUT::AreaInfoService", m_AreaInfoService);

	// Set CORBA Service Ports
	addPort(m_AbsoluteMapServicePort);
	addPort(m_AreaInfoServicePort);

	// Bind variables and configuration variable
	bindParameter("color_reverse", m_color_reverse, "0");
	bindParameter("map_type", m_map_type, "1");
	bindParameter("view_scale", m_view_scale, "1.0");

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// ̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapViewer::onActivated(RTC::UniqueId ec_id){

	// EBhE̐
	cvNamedWindow( WINDOW_NAME, CV_WINDOW_AUTOSIZE );

	if( convert ) delete convert;
	convert = new MapToImage;

	input_robot    = false;
	input_poses    = false;
	input_path     = false;
	input_persons  = false;

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// 񊈐̏
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapViewer::onDeactivated(RTC::UniqueId ec_id){

	// EBhEj
	cvDestroyWindow( WINDOW_NAME );

	delete convert;
	convert = 0;

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// s֐
//-------------------------------------------------------------------------------------------------+
RTC::ReturnCode_t GlobalMapViewer::onExecute(RTC::UniqueId ec_id){

	if( !CheckConnectServicePort() ){

		Sleep( 1000 );
		return RTC::RTC_OK;
	}

	InputRobot();
	InputPoses();
	InputPath();
	InputPersons();

	DrowMap();

	return RTC::RTC_OK;
}


//-------------------------------------------------------------------------------------------------+
// n}̕\
//-------------------------------------------------------------------------------------------------+
void GlobalMapViewer::DrowMap(){

	RTC::OGMapConfig conf = m_AbsoluteMapService->getAbsoluteOGMapConfig();

	if( m_map_type ){

		MRFC::TimedFloatAbsoluteOGMapData *mapdata = m_AbsoluteMapService->getFloatAbsoluteOGMap( conf.origin.position.x, conf.origin.position.y, conf.width, conf.height );

		if( mapdata ){
			convert->DrawMap( mapdata, (bool)m_color_reverse, m_view_scale );
			delete mapdata;
			mapdata = 0;
		}
	}
	else{
		MRFC::TimedAbsoluteOGMapData *mapdata = m_AbsoluteMapService->getAbsoluteOGMap( conf.origin.position.x, conf.origin.position.y, conf.width, conf.height );

		if( mapdata ){
			convert->DrawMap( mapdata, (bool)m_color_reverse, m_view_scale );
			delete mapdata;
			mapdata = 0;
		}
	}

	if( CheckConnectAreaInfoService() ){

		TUT::TimedAreaInfo *areainfo = m_AreaInfoService->getAreaInfo();

		if( areainfo ){
			convert->DrawAreaInfo( areainfo );
			delete areainfo;
			areainfo = 0;
		}
	}

	if( input_persons ) convert->DrawPersons( m_persons );
	if( input_path    ) convert->DrawPath( m_path );
	if( input_poses   ) convert->DrawPoses( m_poses );
	if( input_robot   ) convert->DrawRobot( m_robot_pose.data );

	cvShowImage( WINDOW_NAME, convert->image );
	cvWaitKey(10);
}


//------------------------------------------------------------------------------------------------+
// {bg̓
//------------------------------------------------------------------------------------------------+
void GlobalMapViewer::InputRobot(){

	if( m_robot_poseIn.isNew() ){

		while( !m_robot_poseIn.isEmpty() ) m_robot_poseIn.read();
		input_robot = true;
	}
}


//------------------------------------------------------------------------------------------------+
// ʒupn̓
//------------------------------------------------------------------------------------------------+
void GlobalMapViewer::InputPoses(){

	if( m_posesIn.isNew() ){

		while( !m_posesIn.isEmpty() ) m_posesIn.read();
		input_poses = true;
	}
}


//------------------------------------------------------------------------------------------------+
// oH̓
//------------------------------------------------------------------------------------------------+
void GlobalMapViewer::InputPath(){

	if( m_pathIn.isNew() ){

		while( !m_pathIn.isEmpty() ) m_pathIn.read();
		input_path = true;
	}
}


//------------------------------------------------------------------------------------------------+
// lʒu̓
//------------------------------------------------------------------------------------------------+
void GlobalMapViewer::InputPersons(){

	if( m_personsIn.isNew() ){

		while( !m_personsIn.isEmpty() ) m_personsIn.read();
		input_persons = true;
	}
}


//-------------------------------------------------------------------------------------------------+
// T[rX|[gin}j̐ڑmF
// ڑĂtrueԂ
//-------------------------------------------------------------------------------------------------+
bool GlobalMapViewer::CheckConnectServicePort(){

	if( CORBA::is_nil(m_AbsoluteMapService.getObject()) ){

		cerr << "AbsoluteMap service port is not connected.\n\n";
		return false;
	}

	if( m_AbsoluteMapService.getObject()->_non_existent() ){
	
		cerr << "Service provider of AbsoluteMap is not activated.\n\n";
		return false;
	}

	return true;
}


//-------------------------------------------------------------------------------------------------+
// T[rX|[giAreaInfoj̐ڑmF
// ڑĂtrueԂ
//-------------------------------------------------------------------------------------------------+
bool GlobalMapViewer::CheckConnectAreaInfoService(){

	if( CORBA::is_nil(m_AreaInfoService.getObject()) ) return false;

	if( m_AreaInfoService.getObject()->_non_existent() ) return false;

	return true;
}



//-------------------------------------------------------------------------------------------------+
extern "C"
{
 
  void GlobalMapViewerInit(RTC::Manager* manager)
  {
    coil::Properties profile(GlobalMapViewer_spec);
    manager->registerFactory(profile,
                             RTC::Create<GlobalMapViewer>,
                             RTC::Delete<GlobalMapViewer>);
  }
};
